<?php
/*
 * DateTimeI18NBehavior
 * Automatically converts date and datetime fields to I18N format
 *
 * Author: Ricardo Grana <rickgrana@yahoo.com.br>, <ricardo.grana@pmm.am.gov.br>
 * Version: 1.1
 * Requires: Yii 1.0.9 version
 */

class DateTimeI18NBehavior extends CActiveRecordBehavior
{
	public $dateOutcomeFormat = 'Y-m-d';
	public $dateTimeOutcomeFormat = 'Y-m-d H:i:s';

	public $dateIncomeFormat = 'yyyy-MM-dd';
	public $dateTimeIncomeFormat = 'yyyy-MM-dd hh:mm:ss';

	public $timeFields = array('date', 'datetime', 'timestamp');

	public function beforeSave($event){

		//search for date/datetime columns. Convert it to pure PHP date format
		foreach($event->sender->tableSchema->columns as $columnName => $column){
			if (!in_array($column->dbType, $this->timeFields)) continue;
			if (!strlen($event->sender->$columnName)){
				$event->sender->$columnName = null;
				continue;
			}
			if (($column->dbType == 'date')) {
				$timestamp = CDateTimeParser::parse($event->sender->$columnName, Yii::app()->locale->dateFormat);
				if($timestamp === false) {
					$timestamp = strtotime($event->sender->$columnName);
				}
				$event->sender->$columnName = date($this->dateOutcomeFormat, $timestamp);
			}else{
				$pattern = strtr(Yii::app()->locale->dateTimeFormat, array(
					'{0}' => Yii::app()->locale->timeFormat,
					'{1}' => Yii::app()->locale->dateFormat
				));
				$timestamp = CDateTimeParser::parse($event->sender->$columnName, $pattern);
				if($timestamp === false) {
					$timestamp = strtotime($event->sender->$columnName);
				}
				if($timestamp === false) {
					$timestamp = $event->sender->$columnName;
				}
				if(!$timestamp) return false;
				$event->sender->$columnName = date($this->dateTimeOutcomeFormat, $timestamp);
			}
		}
		return true;
	}

	public function afterFind($event){

		foreach($event->sender->tableSchema->columns as $columnName => $column){
			if (!in_array($column->dbType, $this->timeFields)) continue;

			if (!strlen($event->sender->$columnName)) {
				$event->sender->$columnName = null;
				continue;
			}
			if ($column->dbType == 'date') {
				$timestamp = CDateTimeParser::parse($event->sender->$columnName, $this->dateIncomeFormat);
				if(!$timestamp) return false;
				$event->sender->$columnName = Yii::app()->dateFormatter->formatDateTime($timestamp, 'medium', null);
			} else {
				$timestamp = CDateTimeParser::parse($event->sender->$columnName, $this->dateTimeIncomeFormat);
				if(!$timestamp) return false;
				$event->sender->$columnName = Yii::app()->dateFormatter->formatDateTime($timestamp, 'medium', 'medium');
			}
		}
		return true;
	}
}