(function($) {
	
	/**
	 * Collection of different popups
	 * @class Dialog
	 */
	$.fg.Dialog = {
		
		/**
		 * Similar to JS default alert() function
		 * @param {object} options alert options <br />
		 * <ul>
		 *	<li><b>title</b> {string} - popup title</li>
		 *	<li><b>text</b> {string} - popup message</li>
		 * </ul>
		 * @return {void}
		 */
		alert: function(options)
		{
			var dialog = $( "<div>", {
				html: options.text
			} );
			$( "body" ).append( dialog );
			$( dialog ).dialog( {
				dialogClass: "dialog-alert",
				closeOnEscape: false,
				resizable: false,
				modal: true,
				width: "auto",
				title: options.title
			} );
		},
		
		
		/**
		 * Similar to JS default confirm() function
		 * @param {object} options confirm options <br />
		 * <ul>
		 *	<li><b>title</b> {string} - popup title</li>
		 *	<li><b>text</b> {string} - popup message</li>
		 *	<li><b>ok</b> {function} - callback if pressed "Ok" button</li>
		 *	<li><b>cancel</b> {function} - callback if pressed "Cancel" button</li>
		 * </ul>
		 * @return {void}
		 */
		confirm: function(options)
		{
			var dialog = $( "<div>" )
				.append(
					$( "<span>", {
						style: "float:left; margin:0 7px 20px 0"
					} ).addClass( "ui-icon ui-icon-alert" )
				)
				.append(
					$( "<p>", {html: options.text} )
				)
			var buttons = {};
			buttons[t( "ok" )] = function() {
				$( this ).dialog( "close" );
				if(typeof options.ok === "function") {
					options.ok(this);
				}
			};
			buttons[t( "cancel" )] = function() {
				$( this ).dialog( "close" );
				if(typeof options.cancel === "function") {
					options.cancel(this);
				}
			};
			$( dialog ).dialog({
				resizable: false,
				modal: true,
				buttons: buttons
			});
		},
		
		/**
		 * Similar to JS default prompt() function
		 * @param {object} options prompt options <br />
		 * <ul>
		 *	<li><b>title</b> {string} - popup title</li>
		 *	<li><b>ok</b> {function} - callback if pressed "Ok" button</li>
		 *	<li><b>cancel</b> {function} - callback if pressed "Cancel" button</li>
		 * </ul>
		 * @return {void}
		 */
		prompt: function(options)
		{
			var dialog = $( "<div>" )
				.append(
					$( "<form>" )
						.submit( function(e) {
							e.preventDefault();
							$(this).closest( ".ui-dialog" ).find( "button:first" ).click();
						} )
						.append(
							$( "<input />", {
								type: "text"
							} )
						)
				);
			$( "body" ).append( dialog );
			var buttons = {};
			buttons[t( "ok" )] = function() {
				$( this ).dialog( "close" );
				if(typeof options.ok === "function") {
					options.ok(this, $(this).find( "input" ).val());
				}
			};
			buttons[t( "cancel" )] = function() {
				$( this ).dialog( "close" );
				if(typeof options.cancel === "function") {
					options.cancel(this, $(this).find( "input" ).val());
				}
			};
			$( dialog ).dialog( {
				dialogClass: "dialog-prompt",
				closeOnEscape: false,
				resizable: false,
				modal: true,
				title: options.title,
				buttons: buttons
			} );
		}
		
	};
	
})(jQuery);