(function($) {
	
	/**
	 * The main class of client editor
	 * @class Editor
	 */
	$.fg.Editor = {
			
			/**
			 * params that comes from server side on editor init
			 * @type {object}
			 */
			serverParams: {},
		
		/**
		 * Initialize editor
		 * Include dependent files, and init other modules
		 * @return {void}
		 */
		init: function()
		{
			var bp = $.fg.Config.basePath;
			// thirdparty libs
			$.include( bp+"/assets/js/jquery/jquery-ui/js/jquery-ui-1.8.21.custom.min.js" );
			$.include( bp+"/assets/js/jquery/jquery-ui/css/smoothness/jquery-ui-1.8.21.custom.css" );
			$.include( bp+"/assets/js/jquery/jquery.json-2.3.js" );
			$.include( bp+"/assets/js/jquery/jquery.cookie.js" );
			$.include( bp+"/assets/js/jquery/jquery.class.js" );
			$.include( bp+"/assets/js/jquery/jquery.md5.js" );
			$.include( bp+"/assets/js/jquery/tipsy/jquery.tipsy.js" );
			$.include( bp+"/assets/js/jquery/tipsy/css/tipsy.css" );
			$.include( bp+"/assets/js/jquery/jquery.surfacing.js" );
			$.include( bp+"/assets/js/jquery/toastmessage/javascript/jquery.toastmessage.js" );
			$.include( bp+"/assets/js/jquery/toastmessage/resources/css/jquery.toastmessage.css" );
			
			// fg styles
			$.include( bp+"/assets/css/style.css" );
			$.include( bp+"/assets/css/elements.css" );
			$.include( bp+"/assets/css/fg-form.css" );
			
			// fg core
			$.include( bp+"/assets/js/toolbar.js" );
			$.include( bp+"/assets/js/element_editor.js" );
			$.include( bp+"/assets/js/dialog.js" );
			$.include( bp+"/assets/js/notifier.js" );
			$.include( bp+"/assets/js/elements.js" );
			$.include( bp+"/assets/js/elements_helper.js" );
			$.include( bp+"/assets/js/library.js" );
			$.include( bp+"/assets/js/undoredo.js" );
			
			$.include( bp+"/assets/js/language.js", $.proxy( function()
			{
				// inits
				$.fg.Lang.init($.proxy( function()
				{
					$.include( bp+"/assets/js/validations.js" );
					
					$.fg.Toolbar.init();
					
					// get init params from server
					this.callServerAction( "init", {
						async: false,
						success: function(data)
						{
							$.fg.Editor.serverParams = data;
						}
					} );
					
					$.fg.Library.init();
					$.fg.ElementEditor.init();
					
					// create first form element that will contains all other
					$( "#fg-form" ).append( $.fg.Elements.create( "form" ).render() );
					// fire form afterRender method
					this.getForm().afterRender();
					// test

                                        if ($("#questionaireid").val() != '' && $("#questionaireid").val() != '0'){
                                                $.fg.Editor.callServerAction( "loadForm", {
                                                        success:  function(data) {
                                                                $.fg.Editor.loadFromJSON(data.json);
                                                        },
                                                        data: {id: $("#questionaireid").val()}
                                                } );
                                        } else {
                                                this.getForm().addRow( $.fg.Elements.create( "input" ) );
                                        }

					//this.getForm().addRow( $.fg.Elements.create( "captcha" ) );
				}, this ));
				
			}, this ) );
		},
		
		/**
		 * Save current designed form to the database for publishing
		 * @return {void}
		 */
		save: function()
		{
			$.fg.Dialog.confirm({
				text: t( "save_dialog_confirm" ),
				ok: $.proxy( function(dialog) {
					var id = this.getForm().getProperty( "id" );
					var name = this.getForm().getProperty( "name" );
					var json = this.formToJSON();
					var html = this.formToHTML();
					$("#formoutput").val("<h2>"+name+"</h2>"+html);
					this.callServerAction( "save", {
						data: {
							id: id,
							name: name,
							json: json,
							html: html
						},
						success: function(data)
						{
							$.fg.Notifier.add( t( "save_success" ), "success" );
							$("#questionaireid").val(data.id);
						}
					} );
				}, this )
			});
		},
		
		/**
		 * Save current designed form to the database as template to reuse later
		 * @return {void}
		 */
		saveTemplate: function()
		{
			$.fg.Dialog.prompt({
				title: t( "save_as_template_dialog_title" ),
				ok: $.proxy( function(dialog, inputValue) {
					this.callServerAction( "saveTemplate", {
						data: {name: inputValue, json: this.formToJSON()},
						success: function(data)
						{
							$.fg.Notifier.add( t( "save_as_template_success" ), "success" );
							$.fg.Library.loadLibrary();
						}
					} );
				}, this )
			});
		},
		
		/**
		 * Call server with defined action via ajax
		 * @param {string} actionName the name of the server action
		 * @param {object} options jquery $.ajax options
		 * @return {void}
		 */
		callServerAction: function(actionName, options)
		{
			options = $.extend( {
					type: "POST",
					//dataType: "jsonp",
					dataType: "json",
					success: function(data)
					{
						
					},
					error: function(data, textStatus, errorThrown)
					{
						$.fg.Notifier.add( "<b>"+data.status+" "+textStatus+"</b><br />"+data.responseText, "error" );
					},
					complete: function()
					{
						$.fg.Toolbar.setLoaderActivity( false );
					}
				},
				options || {}
			);
			var actionUrl = $.String.sub( $.fg.Config.serverEditorActionsUrl, {action: actionName} );
			$.fg.Toolbar.setLoaderActivity( true );
			$.ajax( actionUrl, options );
		},
		
		/**
		 * Set user session state
		 * @param {string} name the name of the state
		 * @param {string} value the value of the state
		 * @return {void}
		 */
		setUserState: function(name, value)
		{
			$.cookie(name, value, {path: "/"});
		},
		
		/**
		 * Get user session state
		 * @param {string} name the name of the state
		 * @return {string} user state
		 */
		getUserState: function(name)
		{
			return $.cookie(name);
		},
		
		/**
		 * Pack current designed form to HTML string
		 * @return {string} HTML representation of the form
		 */
		formToHTML: function()
		{
			// use relative appendChild to keep script declarations
			var form = $( "<div>" ).get(0);
			form.appendChild( $( "#fg-form" ).clone().get(0) );
			form = $(form);
			// clear form html
			$.each( $.fg.Elements.storage, function(k, element) {
				element.beforeSaveFormAsHTML && element.beforeSaveFormAsHTML(form);
			} );
			
			return form.html();
		},
		
		/**
		 * Pack current designed form to JSON string
		 * @return {string} JSON representation of the form
		 */
		formToJSON: function()
		{
			return $.toJSON( this._buildElementsObjectsTree(), false, 4 );
		},
		
		/**
		 * Get form element (element that contains all others)
		 * @return {$.fg.ElementsContainerElement} form element
		 */
		getForm: function()
		{
			return $.fg.Elements.getFromStorage( 0 );
		},
		
		/**
		 * Reset current form (clear all elements, properties, etc.)
		 * @return {void}
		 */
		clearEditor: function()
		{
			this.getForm().clear();
		},
		
		/**
		 * Render form from JSON
		 * @param {string} JSON representation of the form
		 * @return {boolean} is form loaded successfully or not
		 */
		loadFromJSON: function(json)
		{
			this.clearEditor();
			if(!json) {
				return false;
			}
			try {
				var object = $.evalJSON( json );
			} catch (e) {
				return false;
			}
			if(!object.type) {
				return false;
			}
			this.getForm().loadFromObject( object );
			this._buildFromObject( object.elements );
			return true;
		},
		
		_buildFromObject: function(elementsArray, elementsContainer)
		{
			elementsContainer = elementsContainer || this.getForm();
			if(elementsArray) {
				$(elementsArray).each( $.proxy( function(i, elementObj) {
					var element = $.fg.Elements.create( elementObj.type );
					element.loadFromObject( elementObj );
					elementsContainer.addRow( element );
					if(element instanceof $.fg.ElementsContainerElement && elementObj.elements && elementObj.elements.length) {
						this._buildFromObject( elementObj.elements, element );
					}
				}, this ) );
			}
		},
		
		_buildElementsObjectsTree: function(elementId, array)
		{
			elementId = elementId || 0;
			var element = $.fg.Elements.getFromStorage( elementId );
			if(typeof array === "undefined") {
				array = [];
			}
			var obj;
			if(elementId === 0) {
				array = element.toObject();
				array.elements = [];
				obj = array;
			} else {
				var index = array.push( element.toObject() ) - 1;
				array[index].elements = [];
				obj = array[index];
			}
			if(element instanceof $.fg.ElementsContainerElement && element.elements.length) {
				for(var i=0, l=element.elements.length; i<l; i++) {
					//alert([element.type, element.id, element.elements, element.elements[i].id]);
					this._buildElementsObjectsTree(element.elements[i].getProperty( "id" ), obj.elements);
				}
			}
			return obj;
		}
		
	};
	
	// register on load
	$( $.proxy( $.fg.Editor.init, $.fg.Editor ) );
	
})(jQuery);
