<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * MA Module
 */
class MarketingAutomationModule extends CWebModule
{
    
    /**
     * The base URL of the Marketing Automation tracker.
     * @var string
     */
    public $markaTrackerBaseUrl = 'http://localhost:8081';
	
    /**
     * The name of the cookie, where the user ID used by Marketing Automation is stored.
     * @var string
     */
	public $markaCookieName = 'markaCookie';
    
    /**
     * The number of seconds, after which cached suggested page infos will expire.
     * Default: 10 minutes
     * @var int 
     */
    public $cacheTime = 600;

    /**
     * init
     *
     * @return void
     */
    public function init()
    {
        parent::init();
    }
    
    /**
     * Public getter for the base URL of the Marketing Automation tracker.
     * @return string
     */
    public function getMarkaTrackerBaseUrl()
    {
        return $this->markaTrackerBaseUrl;
    }
	
    /**
     * Gets the UUID for the user, which identifies the user for Marketing Automation.
     * If the user is logged in, the UUID is read from his profile (and saved to
     * the cookie if it has not been saved there yet), if the user is not logged
     * in, it is read from the cookie.
     * If the user does not have a Marketing Automation ID yet, a new one will 
     * be generated and saved to the cookie and the user's profile, if he is logged in.
     * @return string
     */
	public function getUUID()
    {
		$uuid = '';
		//check if user is logged in and read markaUUID from profile
		//if nothing is set save current UUID or generate new one
		if(!Yii::app()->user->isGuest){					
			$uuid = $this->_getOrSaveUUIDFromProfile();			
		}
		//if no uuid is set, read it from cookie or generate new one
		if ($uuid == ''){			
			$cookieUUID = isset(Yii::app()->request->cookies[$this->markaCookieName]) ? Yii::app()->request->cookies[$this->markaCookieName]->value : '';
			if($cookieUUID != ''){
				$uuid = $cookieUUID;
			}
			else{
				$uuid = $this->_generateUUIDAndSaveToCookie();
			}
		}
		return $uuid;
	}
	
    /**
     * Generates a new UUID, saves it to the cookie and then returns it.
     * @return string
     */
	private function _generateUUIDAndSaveToCookie()
    {
		$uuid = uniqid('',true);
		
		$cookie = new CHttpCookie($this->markaCookieName, $uuid);
		//set cookie
		$cookie->expire = time()+60*60*24*360; 
		Yii::app()->request->cookies[$this->markaCookieName] = $cookie;
		
		return $uuid;
	}
	
    /**
     * Gets the user's UUID from his profile or generates a new one (and saves
     * it to the profile and the cookie) if the user does not have a Marketing Automation ID yet.
     * @return string
     */
	private function _getOrSaveUUIDFromProfile()
    {
		$uuid = '';
		$profile = Profile::model()->findByAttributes(array('user_id' => Yii::app()->user->id));
		if($profile){
			if(isset($profile->markaUUID) && $profile->markaUUID != ''){
				$uuid = $profile->markaUUID;
			}
		}
		if($uuid == ''){
			//marka UUID not saved yet - save it from current cookie or generate new one
			$cookieUUID = isset(Yii::app()->request->cookies[$this->markaCookieName]) ? Yii::app()->request->cookies[$this->markaCookieName]->value : '';
			if($cookieUUID != ''){
				$uuid = $cookieUUID;
				$profile->markaUUID = $cookieUUID;
				$profile->save();
			}
			else{
				$uuid = $this->_generateUUIDAndSaveToCookie();
				$profile->markaUUID = $uuid;
				$profile->save();
			}
		}		
		return $uuid;
	}
}
